Attribute VB_Name = "Common"
Option Explicit
Type RGBWrapper
   Red As Byte
   Green As Byte
   Blue As Byte
   Alpha As Byte
End Type

Type LONGWrapper
   Value As Long
End Type

Type Bitmap
   bmType As Long
   bmWidth As Long
   bmHeight As Long
   bmWidthBytes As Long
   bmPlanes As Integer
   bmBitsPixel As Integer
   bmBits As Long
End Type

Type Point
    x As Double
    y As Double
End Type

Public Const PI As Double = 3.14159265358979

Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, _
    ByVal nCount As Long, ByRef lpObject As Any) As Long
    
Declare Function GetBitmapBits Lib "gdi32" (ByVal hBitmap As Long, _
    ByVal dwCount As Long, ByRef lpBits As Any) As Long
    
Declare Function SetBitmapBits Lib "gdi32" (ByVal hBitmap As Long, _
    ByVal dwCount As Long, ByRef lpBits As Any) As Long

Public Function fpart(ByVal x As Double) As Double
    fpart = x - Int(x)
End Function
Public Function rfpart(ByVal x As Double) As Double
    rfpart = 1 - (x - Int(x))
End Function
Public Function Swap(a, b)
    Dim t
    t = a
    a = b
    b = t
End Function

Public Function ColorToRGB(ByVal clr As Long) As RGBWrapper
    Dim Color As LONGWrapper
    Dim Colors As RGBWrapper
    Color.Value = clr
    LSet Colors = Color
    ColorToRGB = Colors
End Function

Public Sub GetImage(ByVal srcPic As StdPicture, ByRef bArray() As Byte)
    Dim bm As Bitmap
    GetObject srcPic, Len(bm), bm
    ReDim bArray(0 To (bm.bmBitsPixel \ 8) - 1, 0 To bm.bmWidth - 1, 0 To bm.bmHeight - 1)
    
    GetBitmapBits srcPic, bm.bmWidthBytes * bm.bmHeight, bArray(0, 0, 0)
End Sub

Public Sub SetImage(ByRef srcPic As Object, ByRef bArray() As Byte)
    Dim bm As Bitmap
    GetObject srcPic.Image, Len(bm), bm
    SetBitmapBits srcPic.Image, bm.bmWidthBytes * bm.bmHeight, bArray(0, 0, 0)
    
    If srcPic.AutoRedraw = True Then
        srcPic.Picture = srcPic.Image
        srcPic.Refresh
    End If
End Sub

Public Function CullTo(ByVal Inp As Double, ByVal Min As Double, ByVal Max As Double) As Variant
    If Inp < Min Then Inp = Min
    If Inp > Max Then Inp = Max
    CullTo = Inp
End Function

Public Sub Clear(dData() As Byte)
    Dim x As Integer, y As Integer
    For y = 0 To UBound(dData, 3)
        For x = 0 To UBound(dData, 2)
            dData(3, x, y) = 0
            dData(2, x, y) = 0
            dData(1, x, y) = 0
            dData(0, x, y) = 0
        Next x
    Next y
End Sub

Public Sub Crop(dData() As Byte, x As Integer, y As Integer, Width As Integer, Height As Integer)
    Dim tData() As Byte
    Dim tX As Integer, tY As Integer
    
    ReDim tData(0 To 3, 0 To Width - 1, 0 To Height - 1)
    
    For tY = 0 To Height - 1
        For tX = 0 To Width - 1
            If tX + x > 0 And tX + x < UBound(dData, 2) And tY + y > 0 And tY + y < UBound(dData, 3) Then
                tData(3, tX, tY) = dData(3, x + tX, y + tX)
                tData(2, tX, tY) = dData(2, x + tX, y + tX)
                tData(1, tX, tY) = dData(1, x + tX, y + tX)
                tData(0, tX, tY) = dData(0, x + tX, y + tX)
            End If
        Next tX
    Next tY
    dData() = tData()
End Sub

Public Sub Transpose(dData() As Byte)
    Dim x As Integer, y As Integer
    Dim tData() As Byte
    ReDim tData(0 To 3, 0 To UBound(dData, 3), 0 To UBound(dData, 2))
    For y = 0 To UBound(dData, 3)
        For x = 0 To UBound(dData, 2)
            tData(3, y, x) = dData(3, x, y)
            tData(2, y, x) = dData(2, x, y)
            tData(1, y, x) = dData(1, x, y)
            tData(0, y, x) = dData(0, x, y)
        Next x
    Next y
    dData = tData
End Sub

Public Function pointInPolygon(Nodes() As Point, polySides As Integer, ByVal x As Double, ByVal y As Double) As Boolean

    Dim i As Integer, j As Integer
    Dim oddNodes As Boolean
    j = polySides - 1
    oddNodes = False
    For i = 0 To polySides - 1
        If Nodes(i).y < y And Nodes(j).y >= y Or _
         Nodes(j).y < y And Nodes(i).y >= y Then
            If Nodes(i).x + (y - Nodes(i).y) / (Nodes(j).y - Nodes(i).y) * (Nodes(j).x - Nodes(i).x) < x Then
                oddNodes = IIf(oddNodes, False, True)
            End If
        End If
        j = i
    Next i
    pointInPolygon = oddNodes
End Function

Public Sub Rotate(dData() As Byte, angle As Double)
    Dim left As Integer, right As Integer, top As Integer, bottom As Integer
    'left = iif(
End Sub

Public Sub Rotate90(dData() As Byte)
    Dim tData() As Byte
    Dim x As Integer, y As Integer
    ReDim tData(0 To 3, 0 To UBound(dData, 3), 0 To UBound(dData, 2))
    For y = 0 To UBound(dData, 3)
        For x = 0 To UBound(dData, 2)
            tData(3, y, UBound(dData, 2) - x) = dData(3, x, y)
            tData(2, y, UBound(dData, 2) - x) = dData(2, x, y)
            tData(1, y, UBound(dData, 2) - x) = dData(1, x, y)
            tData(0, y, UBound(dData, 2) - x) = dData(0, x, y)
        Next x
    Next y
    dData = tData
End Sub
Public Sub Rotate180(dData() As Byte)
    Dim tData() As Byte
    Dim x As Integer, y As Integer
    ReDim tData(0 To 3, 0 To UBound(dData, 2), 0 To UBound(dData, 3))
    For y = 0 To UBound(tData, 3)
        For x = 0 To UBound(tData, 2)
            tData(3, UBound(dData, 2) - x, UBound(dData, 3) - y) = dData(3, x, y)
            tData(2, UBound(dData, 2) - x, UBound(dData, 3) - y) = dData(2, x, y)
            tData(1, UBound(dData, 2) - x, UBound(dData, 3) - y) = dData(1, x, y)
            tData(0, UBound(dData, 2) - x, UBound(dData, 3) - y) = dData(0, x, y)
        Next x
    Next y
    dData = tData
End Sub
Public Sub Rotate270(dData() As Byte)
    Dim tData() As Byte
    Dim x As Integer, y As Integer
    ReDim tData(0 To 3, 0 To UBound(dData, 3), 0 To UBound(dData, 2))
    For y = 0 To UBound(dData, 3)
        For x = 0 To UBound(dData, 2)
            tData(3, UBound(dData, 3) - y, x) = dData(3, x, y)
            tData(2, UBound(dData, 3) - y, x) = dData(2, x, y)
            tData(1, UBound(dData, 3) - y, x) = dData(1, x, y)
            tData(0, UBound(dData, 3) - y, x) = dData(0, x, y)
        Next x
    Next y
    dData = tData
End Sub
