Attribute VB_Name = "Methods"
Option Explicit
Type RGBWrapper
   Red As Byte
   Green As Byte
   Blue As Byte
   Alpha As Byte
End Type

Private Type LONGWrapper
   Value As Long
End Type

Private Type Bitmap
   bmType As Long
   bmWidth As Long
   bmHeight As Long
   bmWidthBytes As Long
   bmPlanes As Integer
   bmBitsPixel As Integer
   bmBits As Long
End Type

Public Const PI As Double = 3.14159265358979

Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, _
    ByVal nCount As Long, ByRef lpObject As Any) As Long
    
Declare Function GetBitmapBits Lib "gdi32" (ByVal hBitmap As Long, _
    ByVal dwCount As Long, ByRef lpBits As Any) As Long
    
Declare Function SetBitmapBits Lib "gdi32" (ByVal hBitmap As Long, _
    ByVal dwCount As Long, ByRef lpBits As Any) As Long
    
Global Const MinSmooth As Byte = 0
    
Global CurrBlend As Byte
Public Const BlendNormal As Byte = 0
Public Const BlendDifference As Byte = 1
Public Const BlendNegation As Byte = 2
Public Const BlendExclusion As Byte = 3
Public Const BlendScreen As Byte = 4
Public Const BlendAdd As Byte = 5
Public Const BlendDodge As Byte = 6
Public Const BlendLighter As Byte = 7
Public Const BlendMultiply As Byte = 8
Public Const BlendSubtract As Byte = 9
Public Const BlendBurn As Byte = 10
Public Const BlendDarker As Byte = 11
Public Const BlendOverlay As Byte = 12
Public Const BlendSoftLight As Byte = 13
Public Const BlendHardLight As Byte = 14
Public Const BlendReflect As Byte = 15

Private Function roundDown(dblValue As Double) As Double
    On Error GoTo PROC_ERR
    Dim myDec As Long

    myDec = InStr(1, CStr(dblValue), ".", vbTextCompare)
    If myDec > 0 Then
        roundDown = CDbl(Left(CStr(dblValue), myDec))
    Else
        roundDown = dblValue
    End If

PROC_EXIT:
    Exit Function
PROC_ERR:
    MsgBox Err.Description, vbInformation, "Round Down"
End Function

Private Function fpart(ByVal X As Double) As Double
    fpart = X - Int(X)
End Function
Private Function rfpart(ByVal X As Double) As Double
    rfpart = 1 - (X - Int(X))
End Function
Private Function Swap(a, b)
    Dim t
    t = a
    a = b
    b = t
End Function

Private Function ColorToRGB(ByVal clr As Long) As RGBWrapper
    Dim Color As LONGWrapper
    Dim Colors As RGBWrapper
    Color.Value = clr
    LSet Colors = Color
    ColorToRGB = Colors
End Function

Public Sub GetImage(ByVal srcPic As StdPicture, ByRef bArray() As Byte)
    Dim bm As Bitmap
    GetObject srcPic, Len(bm), bm
    ReDim bArray(0 To (bm.bmBitsPixel \ 8) - 1, 0 To bm.bmWidth - 1, 0 To bm.bmHeight - 1)
    
    GetBitmapBits srcPic, bm.bmWidthBytes * bm.bmHeight, bArray(0, 0, 0)
End Sub

Public Sub SetImage(ByRef srcPic As Object, ByRef bArray() As Byte)
    Dim bm As Bitmap
    GetObject srcPic.Image, Len(bm), bm
    SetBitmapBits srcPic.Image, bm.bmWidthBytes * bm.bmHeight, bArray(0, 0, 0)
    
    If srcPic.AutoRedraw = True Then
        srcPic.Picture = srcPic.Image
        srcPic.Refresh
    End If
End Sub

Private Function CullTo(ByVal Inp As Double, ByVal Min As Double, ByVal Max As Double) As Variant
    If Inp < Min Then Inp = Min
    If Inp > Max Then Inp = Max
    CullTo = Inp
End Function

Public Sub AlphaPaintPicture(dData() As Byte, sData() As Byte, Optional ByVal mData, _
                                Optional ByVal DestX As Integer = 0, Optional ByVal DestY As Integer = 0, _
                                Optional ByVal width As Integer = 0, Optional ByVal Height As Integer = 0, _
                                Optional ByVal SrcX As Integer = 0, Optional ByVal SrcY As Integer = 0, _
                                Optional ByVal MaskX As Integer = 0, Optional ByVal MaskY As Integer = 0, _
                                Optional ByVal Alpha As Byte = 255, Optional BlendMode As Byte = BlendNormal)
    'On Error GoTo ErrMsg
    Dim X As Integer, Y As Integer
    Dim dx As Integer, dy As Integer, sx As Integer, sy As Integer, mx As Integer, my As Integer
    Dim rgba As RGBWrapper
    If IsMissing(mData) Then
        mData = sData
        For Y = 0 To UBound(mData, 3)
            For X = 0 To UBound(mData, 2)
                mData(2, X, Y) = 255
                mData(1, X, Y) = 255
                mData(0, X, Y) = 255
            Next X
        Next Y
    End If
    For Y = 0 To Height - 1
        For X = 0 To width - 1
            dx = X + DestX
            dy = Y + DestY
            sx = X + SrcX
            sy = Y + SrcY
            mx = X + MaskX
            my = Y + MaskY
            If mData(2, mx, my) <> 0 Or mData(1, mx, my) <> 0 Or mData(0, mx, my) <> 0 Then
                If mx >= 0 And mx <= UBound(mData, 2) And my >= 0 And my <= UBound(mData, 3) And _
                    sx >= 0 And sx <= UBound(sData, 2) And sy >= 0 And sy <= UBound(sData, 3) Then
                    If dx >= 0 And dx <= UBound(dData, 2) And dy >= 0 And dy <= UBound(dData, 3) Then
                        rgba.Alpha = mData(0, X + MaskX, Y + MaskY)
                        rgba.Red = sData(2, X + SrcX, Y + SrcY)
                        rgba.Green = sData(1, X + SrcX, Y + SrcY)
                        rgba.Blue = sData(0, X + SrcX, Y + SrcY)
                        Blend dData, dx, dy, rgba, (Alpha / 255), BlendMode
                    End If
                End If
            End If
        Next X
    Next Y
    Exit Sub
ErrMsg:
    MsgBox "Could not perform requested operation.", vbExclamation, "Das Paint Methods: Error"
End Sub

Public Sub AlphaCircle(dData() As Byte, ByVal X As Double, ByVal Y As Double, ByVal Radius As Double, Colour As RGBWrapper, Optional ByVal Hardness As Double, Optional BlendMode As Byte = 0, Optional Sinusoidal As Byte = 0)
    'On Error GoTo ErrMsg
    Dim px As Integer, py As Integer
    Dim dist As Double
    Dim diff As Integer
    Dim Multi As Double
    For py = Y - Radius To Y + Radius
        For px = X - Radius To X + Radius
            If py >= 0 And py <= UBound(dData, 3) And px >= 0 And px <= UBound(dData, 2) Then
                If Sinusoidal = 1 Then
                    'dist = Cos(Sqr((X - px) * (X - px) + (Y - py) * (Y - py)) * PI / 2 / Radius)
                    dist = Sqr((X - px) * (X - px) + (Y - py) * (Y - py))
                    If dist < Radius Then
                        dist = (Cos((dist * PI) / Radius) + 1) / 2
                    Else
                        dist = -1
                    End If
                Else
                    dist = 1 - Sqr((X - px) * (X - px) + (Y - py) * (Y - py)) / Radius
                End If
                If dist > 0 Then
                    If dist < (Radius ^ -(Hardness / (100 + MinSmooth))) Then
                        dist = dist / (Radius ^ -(Hardness / (100 + MinSmooth)))
                        'dist = dist / ((Radius * (100 / Hardness)) / 1000)
                    Else
                        dist = 1
                    End If
                    Blend dData, px, py, Colour, dist, BlendMode
                End If
            End If
            
        Next px
    Next py
    
    Exit Sub
ErrMsg:
    MsgBox "Could not perform requested operation.", vbExclamation, "Das Paint Methods: Error"
End Sub

Public Sub AlphaLine(dData() As Byte, ByVal x1 As Double, ByVal y1 As Double, ByVal x2 As Double, ByVal y2 As Double, Colour As RGBWrapper, Optional BlendMode As Byte = 0)
    Dim dx As Double, dy As Double, gradient As Double
    Dim xend As Integer, yend As Integer
    Dim xgap As Double
    Dim xpxl1 As Integer, ypxl1 As Integer
    Dim xpxl2 As Integer, ypxl2 As Integer
    Dim intery As Double
    Dim steep As Byte
    
    Dim X As Integer
    
    If x1 = x2 And y1 = y2 Then
        AlphaPoint dData, x1, y1, Colour, BlendMode
        Exit Sub
    End If
    
    If Abs(x1 - x2) < Abs(y1 - y2) Then
        Swap x1, y2
        Swap x2, y1
        steep = 1
    End If
    
    If x2 < x1 Then
        Swap x1, x2
        Swap y1, y2
    End If

    dx = x2 - x1
    dy = y2 - y1
    gradient = dy / dx

    ' handle first endpoint
    xend = Round(x1)
    yend = y1 + gradient * (xend - x1)
    xgap = rfpart(x1 + 0.5)
    xpxl1 = xend  ' this will be used in the main loop
    ypxl1 = Int(yend)
    If steep = 0 Then
        Blend dData, xpxl1, ypxl1, Colour, rfpart(yend) * xgap, BlendMode
        Blend dData, xpxl1, ypxl1 + 1, Colour, fpart(yend) * xgap, BlendMode
    Else
        Blend dData, ypxl1, xpxl1, Colour, rfpart(yend) * xgap, BlendMode
        Blend dData, ypxl1 + 1, xpxl1, Colour, fpart(yend) * xgap, BlendMode
    End If
    intery = yend + gradient ' first y-intersection for the main loop

    ' handle second endpoint
    xend = Round(x2)
    yend = y2 + gradient * (xend - x2)
    xgap = fpart(x2 + 0.5)
    xpxl2 = xend  ' this will be used in the main loop
    ypxl2 = Int(yend)
    If steep = 0 Then
        Blend dData, xpxl2, ypxl2, Colour, rfpart(yend) * xgap, BlendMode
        Blend dData, xpxl2, ypxl2 + 1, Colour, fpart(yend) * xgap, BlendMode
    Else
        Blend dData, ypxl2, xpxl2, Colour, rfpart(yend) * xgap, BlendMode
        Blend dData, ypxl2 + 1, xpxl2, Colour, fpart(yend) * xgap, BlendMode
    End If

    ' main loop
    If xpxl1 > xpxl2 Then
        xpxl1 = Round(x1)
    End If
    For X = xpxl1 + 1 To xpxl2 - 1
        If steep = 0 Then
            Blend dData, X, Int(intery), Colour, rfpart(intery), BlendMode
            Blend dData, X, Int(intery) + 1, Colour, fpart(intery), BlendMode
        Else
            Blend dData, Int(intery), X, Colour, rfpart(intery), BlendMode
            Blend dData, Int(intery) + 1, X, Colour, fpart(intery), BlendMode
        End If
        intery = intery + gradient
    Next X
End Sub


Public Sub AlphaRectangle(dData() As Byte, ByVal X As Double, ByVal Y As Double, ByVal width As Double, ByVal Height As Double, Colour As RGBWrapper, Optional ByVal Radius As Double = 0, Optional Hardness As Double = 100, Optional ByVal BlendMode As Byte = BlendNormal, Optional Sinusoidal As Byte = 0)
    Dim px As Integer, py As Integer
    Dim dist As Double
    Dim cx As Integer, cy As Integer
    Dim diff As Integer
    
    If Radius > width \ 2 Then
        Radius = width \ 2
    End If
    If Radius > Height \ 2 Then
        Radius = Height \ 2
    End If
    
    For py = CullTo(Y, 0, UBound(dData, 3)) To CullTo(Y + Height - 1, 0, UBound(dData, 3))
        For px = CullTo(X, 0, UBound(dData, 2)) To CullTo(X + width - 1, 0, UBound(dData, 2))
            If (px - X > Radius And px - X < width - Radius) Or _
             (py - Y > Radius And py - Y < Height - Radius) Then
                If py - Y < Radius Then
                    If Sinusoidal = 0 Then
                        dist = 1 - (Radius - (py - Y)) / Radius
                    Else
                        dist = Radius - (py - Y)
                        If dist < Radius Then
                            dist = (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                ElseIf py - Y > Height - Radius Then
                    If Sinusoidal = 0 Then
                        dist = (Height - (py - Y)) / Radius
                    Else
                        dist = Height - (py - Y)
                        If dist < Radius Then
                            dist = 1 - (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                ElseIf px - X < Radius Then
                    If Sinusoidal = 0 Then
                        dist = 1 - (Radius - (px - X)) / Radius
                    Else
                        dist = Radius - (px - X)
                        If dist < Radius Then
                            dist = (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                ElseIf px - X > width - Radius Then
                    If Sinusoidal = 0 Then
                        dist = (width - (px - X)) / Radius
                    Else
                        'dist = Sin((width - (px - X)) * PI / 2 / Radius)
                        dist = width - (px - X)
                        If dist < Radius Then
                            dist = 1 - (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                Else
                    dist = 1
                End If
            Else
                If px - X <= Radius And py - Y <= Radius Then
                    If Sinusoidal = 0 Then
                        dist = 1 - Sqr((px - (X + Radius)) * (px - (X + Radius)) + (py - (Y + Radius)) * (py - (Y + Radius))) / Radius
                    Else
                        dist = Sqr((px - (X + Radius)) * (px - (X + Radius)) + (py - (Y + Radius)) * (py - (Y + Radius)))
                        If dist < Radius Then
                            dist = (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                ElseIf px - X <= Radius And py - Y >= Height - Radius Then
                    If Sinusoidal = 0 Then
                        dist = 1 - Sqr((px - (X + Radius)) * (px - (X + Radius)) + (py - (Y + Height - Radius)) * (py - (Y + Height - Radius))) / Radius
                    Else
                        dist = Sqr((px - (X + Radius)) * (px - (X + Radius)) + (py - (Y + Height - Radius)) * (py - (Y + Height - Radius)))
                        If dist < Radius Then
                            dist = (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                ElseIf px - X >= width - Radius And py - Y <= Radius Then
                    If Sinusoidal = 0 Then
                        dist = 1 - Sqr((px - (X + width - Radius)) * (px - (X + width - Radius)) + (py - (Y + Radius)) * (py - (Y + Radius))) / Radius
                    Else
                        dist = Sqr((px - (X + width - Radius)) * (px - (X + width - Radius)) + (py - (Y + Radius)) * (py - (Y + Radius)))
                        If dist < Radius Then
                            dist = (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                ElseIf px - X >= width - Radius And py - Y >= Height - Radius Then
                    If Sinusoidal = 0 Then
                        dist = 1 - Sqr((px - (X + width - Radius)) * (px - (X + width - Radius)) + (py - (Y + Height - Radius)) * (py - (Y + Height - Radius))) / Radius
                    Else
                        dist = Sqr((px - (X + width - Radius)) * (px - (X + width - Radius)) + (py - (Y + Height - Radius)) * (py - (Y + Height - Radius)))
                        If dist < Radius Then
                            dist = (Cos((dist * PI) / Radius) + 1) / 2
                        Else
                            dist = -1
                        End If
                    End If
                End If
            End If
            If dist < (Radius ^ -(Hardness / (100 + MinSmooth))) Then
                dist = dist / (Radius ^ -(Hardness / (100 + MinSmooth)))
            Else
                dist = 1
            End If
            If dist > 0 Then
                Blend dData(), px, py, Colour, dist, BlendMode
            End If
        Next px
    Next py
End Sub

Public Sub AlphaPoint(dData() As Byte, ByVal X As Double, ByVal Y As Double, Colour As RGBWrapper, BlendMode As Byte)
    Blend dData(), Int(X), Int(Y), Colour, rfpart(X) * rfpart(Y), BlendMode
    Blend dData(), Int(X) + 1, Int(Y), Colour, fpart(X) * rfpart(Y), BlendMode
    Blend dData(), Int(X), Int(Y) + 1, Colour, rfpart(X) * fpart(Y), BlendMode
    Blend dData(), Int(X) + 1, Int(Y) + 1, Colour, fpart(X) * fpart(Y), BlendMode
End Sub

Private Sub Blend(dData() As Byte, X As Integer, Y As Integer, FG As RGBWrapper, Optional Alpha As Double = 1, Optional Mode As Byte = BlendNormal)
    Dim diff As Integer
    Dim fRed As Long, fGreen As Long, fBlue As Long
    Dim bRed As Long, bGreen As Long, bBlue As Long
    If X < 0 Or X > UBound(dData, 2) Or Y < 0 Or Y > UBound(dData, 3) Or Alpha = 0 Or FG.Alpha = 0 Then
        Exit Sub
    End If
    bRed = dData(2, X, Y)
    bGreen = dData(1, X, Y)
    bBlue = dData(0, X, Y)
    
    fRed = FG.Red
    fGreen = FG.Green
    fBlue = FG.Blue
    Select Case Mode
        Case BlendNormal:
            diff = (fRed - bRed) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed + diff, 0, 255)
            diff = (fGreen - bGreen) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen + diff, 0, 255)
            diff = (fBlue - bBlue) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue + diff, 0, 255)
        Case BlendDifference:
            diff = Abs(bRed - fRed)
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            diff = Abs(bGreen - fGreen)
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            diff = Abs(bBlue - fBlue)
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendNegation:
            diff = 255 - Abs(255 - bRed - fRed)
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            diff = 255 - Abs(255 - bGreen - fGreen)
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            diff = 255 - Abs(255 - bBlue - fBlue)
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendExclusion:
            diff = fRed + bRed - (fRed * bRed \ 128)
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            diff = fGreen + bGreen - (fGreen * bGreen \ 128)
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            diff = fBlue + bBlue - (fBlue * bBlue \ 128)
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendScreen:
            diff = 255 - ((255 - fRed) * (255 - bRed) \ 255)
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            diff = 255 - ((255 - fGreen) * (255 - bGreen) \ 255)
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            diff = 255 - ((255 - fBlue) * (255 - bBlue) \ 255)
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendDodge:
            If bRed = 255 Then
                diff = 255
            Else
                diff = CullTo((fRed * 255) \ (255 - bRed), 0, 255)
            End If
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            If bGreen = 255 Then
                diff = 255
            Else
                diff = CullTo((fGreen * 255) \ (255 - bGreen), 0, 255)
            End If
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            If bBlue = 255 Then
                diff = 255
            Else
                diff = CullTo((fBlue * 255) \ (255 - bBlue), 0, 255)
            End If
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendAdd:
            diff = fRed * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed + diff, 0, 255)
            diff = fGreen * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen + diff, 0, 255)
            diff = fBlue * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue + diff, 0, 255)
        Case BlendLighter:
            If fRed > bRed Then
                diff = (bRed - fRed) * Alpha * (FG.Alpha / 255)
                bRed = CullTo(bRed - diff, 0, 255)
            End If
            If fGreen > bGreen Then
                diff = (bGreen - fGreen) * Alpha * (FG.Alpha / 255)
                bGreen = CullTo(bGreen - diff, 0, 255)
            End If
            If fBlue > bBlue Then
                diff = (bBlue - fBlue) * Alpha * (FG.Alpha / 255)
                bBlue = CullTo(bBlue - diff, 0, 255)
            End If
        Case BlendMultiply:
            diff = (fRed * bRed) \ 255
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            diff = (fGreen * bGreen) \ 255
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            diff = (fBlue * bBlue) \ 255
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendBurn:
            If bRed = 0 Then
                diff = 0
            Else
                diff = CullTo(255 - (((255 - fRed) * 255) \ bRed), 0, 255)
            End If
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            If bGreen = 0 Then
                diff = 0
            Else
                diff = CullTo(255 - (((255 - fGreen) * 255) \ bGreen), 0, 255)
            End If
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            If bBlue = 0 Then
                diff = 0
            Else
                diff = CullTo(255 - (((255 - fBlue) * 255) \ bBlue), 0, 255)
            End If
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendSubtract:
            diff = bRed - fRed
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            diff = bGreen - fGreen
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            diff = bBlue - fBlue
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendDarker:
            If fRed < bRed Then
                diff = (bRed - fRed) * Alpha * (FG.Alpha / 255)
                bRed = CullTo(bRed - diff, 0, 255)
            End If
            If fGreen < bGreen Then
                diff = (bGreen - fGreen) * Alpha * (FG.Alpha / 255)
                bGreen = CullTo(bGreen - diff, 0, 255)
            End If
            If fBlue < bBlue Then
                diff = (bBlue - fBlue) * Alpha * (FG.Alpha / 255)
                bBlue = CullTo(bBlue - diff, 0, 255)
            End If
        Case BlendOverlay:
            If fRed < 128 Then
                diff = (fRed * bRed) \ 128
            Else
                diff = 255 - ((255 - fRed) * (255 - bRed) \ 128)
            End If
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            If fGreen < 128 Then
                diff = (fGreen * bGreen) \ 128
            Else
                diff = 255 - ((255 - fGreen) * (255 - bGreen) \ 128)
            End If
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            If fBlue < 128 Then
                diff = (fBlue * bBlue) \ 128
            Else
                diff = 255 - ((255 - fBlue) * (255 - bBlue) \ 128)
            End If
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendSoftLight:
            diff = fRed * bRed \ 255
            diff = diff + fRed * (255 - ((255 - fRed) * (255 - bRed) \ 255) - diff) \ 255
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            diff = fGreen * bGreen \ 255
            diff = diff + fGreen * (255 - ((255 - fGreen) * (255 - bGreen) \ 255) - diff) \ 255
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            diff = fBlue * bBlue \ 255
            diff = diff + fBlue * (255 - ((255 - fBlue) * (255 - bBlue) \ 255) - diff) \ 255
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendHardLight:
            If bRed < 128 Then
                diff = (fRed * bRed) \ 128
            Else
                diff = 255 - ((255 - bRed) * (255 - fRed) \ 128)
            End If
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            If bGreen < 128 Then
                diff = (fGreen * bGreen) \ 128
            Else
                diff = 255 - ((255 - bGreen) * (255 - fGreen) \ 128)
            End If
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            If bBlue < 128 Then
                diff = (fBlue * bBlue) \ 128
            Else
                diff = 255 - ((255 - bBlue) * (255 - fBlue) \ 128)
            End If
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
        Case BlendReflect:
            If bRed = 255 Then
                diff = 255
            Else
                diff = CullTo(fRed * fRed \ (255 - bRed), 0, 255)
            End If
            diff = (bRed - diff) * Alpha * (FG.Alpha / 255)
            bRed = CullTo(bRed - diff, 0, 255)
            
            If bGreen = 255 Then
                diff = 255
            Else
                diff = CullTo(fGreen * fGreen \ (255 - bGreen), 0, 255)
            End If
            diff = (bGreen - diff) * Alpha * (FG.Alpha / 255)
            bGreen = CullTo(bGreen - diff, 0, 255)
            
            If bBlue = 255 Then
                diff = 255
            Else
                diff = CullTo(fBlue * fBlue \ (255 - bBlue), 0, 255)
            End If
            diff = (bBlue - diff) * Alpha * (FG.Alpha / 255)
            bBlue = CullTo(bBlue - diff, 0, 255)
    End Select
    dData(2, X, Y) = bRed
    dData(1, X, Y) = bGreen
    dData(0, X, Y) = bBlue
End Sub


Public Sub Transpose(dData() As Byte)
    Dim X As Integer, Y As Integer
    Dim tdata() As Byte
    Dim rgb As RGBWrapper
    ReDim tdata(0 To UBound(dData, 1), 0 To UBound(dData, 3), 0 To UBound(dData, 2))
    For Y = 0 To UBound(dData, 3)
        For X = 0 To UBound(dData, 2)
            rgb.Alpha = dData(3, X, Y)
            rgb.Red = dData(2, X, Y)
            rgb.Green = dData(1, X, Y)
            rgb.Blue = dData(0, X, Y)
            
            
            tdata(3, Y, X) = rgb.Alpha
            tdata(2, Y, X) = rgb.Red
            tdata(1, Y, X) = rgb.Green
            tdata(0, Y, X) = rgb.Blue
        Next X
    Next Y
    dData = tdata
End Sub

Public Sub hblur(dData() As Byte, ByVal lblur As Double)

    Dim newIm() As Byte
    Dim X As Integer, Y As Integer
    Dim bx As Integer
    Dim aR As Double, aG As Double, aB As Double, aA As Double
    Dim i As Integer
    
    Dim ave As Double
    
    newIm() = dData()
    
    For Y = 0 To UBound(dData, 3)
        For X = 0 To UBound(dData, 2)
            aR = 0
            aG = 0
            aB = 0
            ave = 0
            i = 0
            For bx = CullTo(X - lblur, 0, UBound(dData, 2)) To CullTo(X + lblur, 0, UBound(dData, 2))
                aA = aA + dData(3, bx, Y)
                aR = aR + dData(2, bx, Y)
                aG = aG + dData(1, bx, Y)
                aB = aB + dData(0, bx, Y)
                i = i + 1
            Next bx
'            aR = aR - CullTo(x - lblur, 0, UBound(dData, 2)) + CullTo(x + lblur, 0, UBound(dData, 2))
'            aG = aG - CullTo(x - lblur, 0, UBound(dData, 2)) + CullTo(x + lblur, 0, UBound(dData, 2))
'            aB = aB - CullTo(x - lblur, 0, UBound(dData, 2)) + CullTo(x + lblur, 0, UBound(dData, 2))
            
            ave = aR / i
            newIm(2, X, Y) = CullTo(ave, 0, 255)
            
            ave = aG / i
            newIm(1, X, Y) = CullTo(ave, 0, 255)
            
            ave = aB / i
            newIm(0, X, Y) = CullTo(ave, 0, 255)
            
            ave = aA / i
            newIm(3, X, Y) = CullTo(ave, 0, 255)
        Next X
        If Y Mod 5 = 0 Then
            DoEvents
        End If
    Next Y
    dData = newIm

End Sub

Public Sub BoxBlur(dData() As Byte, ByVal lblur As Double)
    
    hblur dData(), lblur
    Transpose dData()
    hblur dData(), lblur
    Transpose dData()
End Sub

'Public Sub Edge(dData() As Byte, ByVal Size As Double)
'    Grayscale dData()
'    SetImage frmMain.imgDest, dData()
'    BoxBlur dData(), Size
'    AlphaPaintPicture dData, frmMain.imgDest.Image, , , , frmMain.imgDest.ScaleWidth, frmMain.imgDest.ScaleHeight, , , , , , BlendSubtract
'End Sub

Public Sub Grayscale(dData() As Byte)
    Dim X As Integer, Y As Integer
    Dim r As Integer, g As Integer, b As Integer
    For Y = 0 To UBound(dData, 3)
        For X = 0 To UBound(dData, 2)
            r = dData(2, X, Y)
            g = dData(1, X, Y)
            b = dData(0, X, Y)
            dData(2, X, Y) = (r + g + b) \ 3
            dData(1, X, Y) = dData(2, X, Y)
            dData(0, X, Y) = dData(2, X, Y)
        Next X
    Next Y
End Sub


Public Sub Rescale(dData() As Byte, ByVal width As Integer, ByVal Height As Integer)
    If width <> 0 And Height <> 0 Then
        BiLinear dData, width
        Transpose dData
        BiLinear dData, Height
        Transpose dData
    End If
End Sub

Private Sub BiLinear(dData() As Byte, width As Integer)
    Dim tdata() As Byte
    Dim X As Integer, Y As Integer
    Dim rgb1 As RGBWrapper, rgb2 As RGBWrapper
    Dim rgbt As RGBWrapper
    Dim off As Double, dist As Double
    Dim ioff As Integer
    dist = width / UBound(dData, 2)
    ReDim tdata(0 To UBound(dData, 1), 0 To width - 1, 0 To UBound(dData, 3))
    For Y = 0 To UBound(tdata, 3)
        For X = 0 To width - 1
            ioff = X * dist
            off = X * dist
            rgb1.Red = dData(0, off, Y)
            rgb1.Green = dData(1, off, Y)
            rgb1.Blue = dData(2, off, Y)
            If off + 1 <> UBound(tdata, 2) + 1 Then
                rgb2.Red = dData(0, off + 1, Y)
                rgb2.Green = dData(1, off + 1, Y)
                rgb2.Blue = dData(2, off + 1, Y)
            End If
            
            off = off - ioff + dist
            tdata(0, X, Y) = (rgb1.Red + rgb2.Red * off) / 2
            'tdata(0, x, y)
        Next X
    Next Y
End Sub
