Attribute VB_Name = "QuickMethods"
Option Explicit

Public Sub QuickBlend(dData() As Byte, x As Integer, y As Integer, Colour As RGBWrapper)
    Dim diff As Double
    Dim cr As Integer, cg As Integer, cb As Integer
    If x >= 0 And x <= UBound(dData, 2) And y >= 0 And y <= UBound(dData, 3) Then
        cr = Colour.Red: cg = Colour.Green: cb = Colour.Blue
        diff = (cr - dData(2, x, y)) * (Colour.Alpha / 255)
        dData(2, x, y) = dData(2, x, y) + diff
        diff = (cg - dData(1, x, y)) * (Colour.Alpha / 255)
        dData(1, x, y) = dData(1, x, y) + diff
        diff = (cb - dData(0, x, y)) * (Colour.Alpha / 255)
        dData(0, x, y) = dData(0, x, y) + diff
    End If
End Sub

Public Sub QuickPaintPicture(dData() As Byte, sData() As Byte, _
                                ByVal DestX As Integer, ByVal DestY As Integer, _
                                ByVal Width As Integer, ByVal Height As Integer, _
                                MaskColour As RGBWrapper, _
                                Optional ByVal SrcX As Integer = 0, Optional ByVal SrcY As Integer = 0, _
                                Optional ByVal Alpha As Byte = 255)
    'On Error GoTo ErrMsg
    Dim x As Integer, y As Integer
    Dim dX As Integer, dY As Integer, sx As Integer, sy As Integer, mx As Integer, my As Integer
    Dim rgba As RGBWrapper
    For y = 0 To Height - 1
        For x = 0 To Width - 1
            dX = x + DestX
            dY = y + DestY
            sx = x + SrcX
            sy = y + SrcY
            If sx >= 0 And sx <= UBound(sData, 2) And sy >= 0 And sy <= UBound(sData, 3) Then
                If dX >= 0 And dX <= UBound(dData, 2) And dY >= 0 And dY <= UBound(dData, 3) Then
                    rgba.Red = sData(2, sx, sy)
                    rgba.Green = sData(1, sx, sy)
                    rgba.Blue = sData(0, sx, sy)
                    rgba.Alpha = Alpha
                    If rgba.Red <> MaskColour.Red Or rgba.Green <> MaskColour.Green Or _
                     rgba.Blue <> MaskColour.Blue Then
                        QuickBlend dData, dX, dY, rgba
                    End If
                End If
            End If
        Next x
    Next y
    Exit Sub
ErrMsg:
    MsgBox "Could not perform requested operation.", vbExclamation, "Das Paint Methods: Error"
End Sub

Public Sub QuickPaintColour(dData() As Byte, mData() As Byte, Colour As RGBWrapper, _
                                ByVal DestX As Integer, ByVal DestY As Integer, _
                                ByVal Width As Integer, ByVal Height As Integer, _
                                MaskColour As RGBWrapper, _
                                Optional ByVal MaskX As Integer = 0, Optional ByVal MaskY As Integer = 0, _
                                Optional ByVal Alpha As Byte = 255)
    On Error GoTo ErrMsg
    Dim x As Integer, y As Integer
    Dim dX As Integer, dY As Integer, mx As Integer, my As Integer
    Dim rgba As RGBWrapper
    Dim diff As Double
    For y = 0 To Height - 1
        For x = 0 To Width - 1
            dX = x + DestX
            dY = y + DestY
            If mData(2, x, y) <> MaskColour.Red Or mData(1, x, y) <> MaskColour.Green Or mData(0, x, y) <> MaskColour.Blue Then
                Colour.Alpha = mData(2, x, y)
                QuickBlend dData, dX, dY, Colour
            End If
        Next x
    Next y
    Exit Sub
ErrMsg:
    MsgBox "Could not perform requested operation.", vbExclamation, "Das Paint Methods: Error"
End Sub

Public Sub QuickPoly(dData() As Byte, Nodes() As Point, Colour As RGBWrapper, _
                        Optional ByVal DestX As Integer = 1, Optional ByVal DestY As Integer = 0, _
                        Optional ByVal Alpha As Byte = 255)
    Dim x As Integer, y As Integer, i As Integer, j As Integer
    Dim top As Integer, left As Integer, right As Integer, bottom As Integer
    Colour.Alpha = Alpha
    top = Nodes(0).y
    bottom = top
    left = Nodes(0).x
    right = left
    For i = 0 To UBound(Nodes)
        If Nodes(i).y < top Then top = Nodes(i).y
        If Nodes(i).y > bottom Then bottom = Nodes(i).y
        If Nodes(i).x < left Then left = Nodes(i).x
        If Nodes(i).x > right Then right = Nodes(i).x
    Next i
    For y = top To bottom
        For x = left To right
            If x + DestX > 0 And x + DestX < UBound(dData, 2) And _
             y + DestY > 0 And y + DestY < UBound(dData, 3) Then
                If pointInPolygon(Nodes, UBound(Nodes) + 1, x, y) Then
                    QuickBlend dData, x + DestX, y + DestY, Colour
                End If
            End If
        Next x
    Next y

End Sub

Public Sub QuickRectangle(dData() As Byte, ByVal x As Integer, ByVal y As Integer, ByVal Width As Integer, ByVal Height As Integer, Colour As RGBWrapper)
    Dim px As Integer, py As Integer
    For py = CullTo(y, 0, UBound(dData, 3)) To CullTo(y + Height - 1, 0, UBound(dData, 3))
        For px = CullTo(x, 0, UBound(dData, 2)) To CullTo(x + Width - 1, 0, UBound(dData, 2))
            QuickBlend dData(), px, py, Colour
        Next px
    Next py
End Sub
